@@ -29,6 +29,7 @@ def remove_unique_id(response: dict) -> dict:
                  remove_unique_id(value)
     return response
 
+
 class BottleClient:
     """Interface for interacting with bottles."""
     def __init__(self, bottle_info):
@@ -60,12 +61,11 @@ class BottleClient:
         self.headers = {'X-API-KEY': self.api_key}
         self.all_nodes = []
         self.failures = []
-        self.system_failures = []
         self._connected = False
         self.genome = None
         self.genie = None
         self.send_unique_ids = True
-        self.summarize_for_single_node = False
+        self.summarize_for_single_node = True
         if 'secure' not in self.bottle_info or self.bottle_info['secure']:
             self.secure = True
             self.url = 'https://{name}.{domain}/'.format(**self.bottle_info)
@@ -118,7 +118,7 @@ class BottleClient:
         """Internal helper function to make an RPC call with the given *query* and *data*."""
         if not self._connected:
             raise ConnectionError('Not connected to a bottle. You must call `connect()` on a BottleClient instance before making queries')
-        result = []
+        result = {}
         if unique_id is not None:
             if data:
                     data['unique_id'] = unique_id
@@ -143,10 +143,10 @@ class BottleClient:
                     response = remove_unique_id(response['message'])
                 else:
                     response = response['message']
-                if self.summarize_for_single_node and len(nodes) == 1:
+                if len(nodes) == 1 and self.summarize_for_single_node:
                     result = response
                 else:
-                    result.append({node['name']: response})
+                    result[node['name']] = response
             except Exception as exception:
                 self.failures.append({node['name']: {'class': exception.__class__.__name__, 'message': str(exception)}})
                 raise QueryError("Query Failure:", {'class': exception.__class__.__name__, 'message': str(exception)}) from exception
@@ -165,12 +165,14 @@ class BottleClient:
 
     def observe_event(self, data: Dict = None, unique_id: str = None) -> List:
         """Exclusively uses the 'observe' call.  All commands must be provided via Genie Metalanguage data."""
-        results = []
+        results = {}
         if unique_id is None:
             unique_id = str(uuid.uuid4())
         for node, node_data in data.items():
-            result, uid = self._query(requests.post, 'observe', node_data, nodes=[node], unique_id=unique_id)
+            response, uid = self._query(requests.post, 'observe', node_data, nodes=[node], unique_id=unique_id)
             results.append(result)
+            response['unique_id'] = uid
+            results[node['name']] = response
         if len(results) == 1:
             results = results[0]
         return results, uid
@@ -241,12 +243,6 @@ class BottleClient:
             nodes = self.query_nodes
         return self._query(requests.get, 'cognition-data', nodes=nodes)
 
-    def get_cogitated(self, nodes: List = None) -> List:
-        """Return cogitated data."""
-        if nodes is None:
-            nodes = self.query_nodes
-        return self._query(requests.get, 'cogitated', nodes=nodes)
-
     def get_decision_table(self, nodes: List = None) -> List:
         """Return a decision table."""
         if nodes is None:
@@ -259,29 +255,37 @@ class BottleClient:
             nodes = self.query_nodes
         return self._query(requests.get, 'action-data', nodes=nodes)
 
-    def change_genes(self, gene_data: Dict) -> List:
+    def change_genes(self, gene_data: Dict, nodes: List = None) -> List:
         """
         Use primitive names.
         This will do live updates to an existing agent, rather than stopping an agent and starting a new one as per 'injectGenome'.
         gene_data of form:
 
-            {node-name: {gene: value}}
-
-        where node-id is the ID of a primitive or manipulative.
+            {gene: value}
 
         Only works on primitive nodes at this time.
         """
         if not self._connected:
             raise ConnectionError('Not connected to a bottle. You must call `connect()` on a BottleClient instance before making queries')
-        self.genome.change_genes(gene_data)
-        result = []
-        for node, updates in gene_data.items():  ## only primitive nodes at this time.
-            for gene, value in updates.items():
-                response = requests.post(f'{self.url}{self.genome.primitive_map[node]}/genome/update', verify=self.secure, headers=self.headers, json={'data': {gene: value}}).json()
-                if 'error' in response or response["message"] != 'updated-genes':
-                    self.system_failures.append({node: response})
-                    print("System Failure:", {node: response})
-                result.append({node: response["message"]})
+        if nodes is None:
+            nodes = self.all_nodes
+        else:
+            nodes = [node for node in self.all_nodes if (node['name'] in nodes)]
+        
+        result = {}
+        for node in nodes:  ## only primitive nodes at this time.
+            response = requests.post(f"{self.url}{node['id']}/genes/change", 
+                                        verify=self.secure, headers=self.headers, 
+                                        json={'data': gene_data}).json()
+            if 'error' in response or response['status'] == 'failed':
+                if len(nodes) == 1 and self.summarize_for_single_node:
+                    raise Exception("Request Failure:", response)
+                self.failures.append({node['name']: exception})
+                print("Failure:", {node['name']: response})
+            self.genome.change_genes(node['id'], gene_data)
+            if len(nodes) == 1 and self.summarize_for_single_node:
+                return response['message']
+            result[node['name']] = response['message']
         return result
 
     def inject_genome(self, genome: Dict) -> Dict:
@@ -306,34 +310,44 @@ class BottleClient:
             nodes = self.query_nodes
         return self._query('getCostBenefit', nodes=nodes)
 
-    def get_gene(self, gene: Dict, nodes: List = None) -> List[Dict[str, Dict[str, str]]]:
+    def get_gene(self, gene: str, nodes: List = None) -> List[Dict[str, Dict[str, str]]]:
         """
         Use primitive names.
-        This will return the gene-value of an existing agent, gene_data of form (similar to that
+        This will return the gene value of an existing agent, gene_data of form (similar to that
         of the change_genes method):
 
-            {node-name: gene}
+            gene
 
-        where node-name is the ID of a primitive or manipulative.
+        where gene is the name of the gene.
 
         Only works on primitive nodes at this time.
         """
         if not self._connected:
             raise ConnectionError('Not connected to a bottle. You must call `connect()` on a BottleClient instance before making queries')
-        result = []
-        for node_name, attribute in gene.items():
+        if nodes is None:
+            nodes = self.all_nodes
+        else:
+            nodes = [node for node in self.all_nodes if (node['name'] in nodes)]
+        result = {}
+        for node in nodes:
             try:
-                response = requests.get(f'{self.url}{self.genome.primitive_map[node_name]}/gene/{attribute}', verify=self.secure,
-                                        headers=self.headers).json()['message']
-                result.append({node_name: {attribute : response}})
-
+                response = requests.get(f"{self.url}{node['id']}/gene/{gene}", 
+                                        verify=self.secure,
+                                        headers=self.headers).json()
+                if 'error' in response or response['status'] == 'failed':
+                    if len(nodes) == 1 and self.summarize_for_single_node:
+                        raise Exception("Request Failure:", response)
+                    self.failures.append({node['name']: exception})
+                if len(nodes) == 1 and self.summarize_for_single_node:
+                    return {gene : response['message']}
+                result[node['name']] = {gene : response['message']}
             except Exception as exception:
-                self.failures.append({node_name: exception})
-                raise Exception("Query Failure:", {node_name: exception})
+                self.failures.append({node['name']: exception})
+                raise Exception("Request Failure:", {node['name']: exception})
 
         return result
 
-    def get_model(self, data: Dict, nodes: List = None) -> List[Dict[str, Dict[str, str]]]:
+    def get_model(self, model_name: Dict, nodes: List = None) -> List[Dict[str, Dict[str, str]]]:
         """
         Returns model.
 
@@ -345,15 +359,27 @@ class BottleClient:
         """
         if not self._connected:
             raise ConnectionError('Not connected to a bottle. You must call `connect()` on a BottleClient instance before making queries')
-        result = []
-        for node_name, model_name in data.items():
+        if nodes is None:
+            nodes = self.all_nodes
+        else:
+            nodes = [node for node in self.all_nodes if (node['name'] in nodes)]
+        result = {}
+        for node in nodes:
             try:
-                response = requests.get(f'{self.url}{self.genome.primitive_map[node_name]}/model/{model_name}', headers=self.headers, verify=self.secure).json()['message']
-                result.append({node_name: {model_name : response}})
-
+                response = requests.get(f"{self.url}{node['id']}/model/{model_name}", 
+                                        headers=self.headers, 
+                                        verify=self.secure).json()
+                if 'error' in response or response['status'] == 'failed':
+                    if len(nodes) == 1 and self.summarize_for_single_node:
+                        raise Exception("Request Failure:", response)
+                    self.failures.append({node['name']: exception})
+                if len(nodes) == 1 and self.summarize_for_single_node:
+                    return response['message']
+                else:
+                    result[node['name']] = response['message']
             except Exception as exception:
-                self.failures.append({node_name: exception})
-                raise Exception("Query Failure:", {node_name: exception})
+                self.failures.append({node['name']: exception})
+                raise Exception("Request Failure:", {node['name']: exception})
 
         return result
 
@@ -371,40 +397,59 @@ class BottleClient:
             nodes = self.all_nodes
         return self._query(requests.get, 'time', nodes=nodes)
 
-    def get_vector(self, data: Dict) -> List:
+    def get_vector(self, vector_name: str, nodes: List = None) -> List:
         """
         """
         if not self._connected:
             raise ConnectionError('Not connected to a bottle. You must call `connect()` on a BottleClient instance before making queries')
-        result = []
-        for node_name, vector_name in data.items():
+        if nodes is None:
+            nodes = self.all_nodes
+        else:
+            nodes = [node for node in self.all_nodes if (node['name'] in nodes)]
+        result = {}
+        for node in nodes:
             try:
-                response = requests.post(self.url, verify=self.secure, json={"method": "getVector", "params": {"api_key": self.api_key, "primitive_id": self.genome.primitive_map[node_name], "data": vector_name},
+                response = requests.post(self.url, verify=self.secure, json={"method": "getVector", "params": {"api_key": self.api_key, "primitive_id": node['id'], "data": vector_name},
                      "jsonrpc": "2.0", "id": 1}).json()['result']
-                result.append({node_name: {vector_name: response}})
-
+                if 'error' in response or response['status'] == 'failed':
+                    if len(nodes) == 1 and self.summarize_for_single_node:
+                        raise Exception("Request Failure:", response)
+                    self.failures.append({node['name']: exception})
+                if len(nodes) == 1 and self.summarize_for_single_node:
+                    return {vector_name: response}
+                result[node['name']] = {vector_name: response}
             except Exception as exception:
-                self.failures.append({node_name: exception})
-                raise Exception("Query Failure:", {node_name: exception})
-
+                self.failures.append({node['name']: exception})
+                raise
         return result
 
-    def increment_recall_threshold(self, data: Dict) -> List[Dict[str, Any]]:
+    def increment_recall_threshold(self, increment: float, nodes: List = None) -> List[Dict[str, Any]]:
         """
-        TODO: update local genome
-        TODO: Change param name 'data' -> 'increment'
         """
         if not self._connected:
             raise ConnectionError('Not connected to a bottle. You must call `connect()` on a BottleClient instance before making queries')
-        result = []
-        for node_name, val in data.items():
+        if nodes is None:
+            nodes = self.all_nodes
+        else:
+            nodes = [node for node in self.all_nodes if (node['name'] in nodes)]
+        result = {}
+        for node in nodes:
             try:
-                response = requests.post(f'{self.url}{self.genome.primitive_map[node_name]}/gene/increment-recall-threshold', verify=self.secure, headers=self.headers, json={'data': val}).json()['message']
-                result.append({node_name: {"recall_threshold": response}})
-
+                response = requests.post(f"{self.url}{node['id']}/gene/increment-recall-threshold", 
+                                            verify=self.secure, headers=self.headers, 
+                                            json={'increment': increment}).json()
+                if 'error' in response or response['status'] == 'failed':
+                    if len(nodes) == 1 and self.summarize_for_single_node:
+                        raise Exception("Request Failure:", response)
+                    self.failures.append({node['name']: exception})
+                else:
+                    self.genome.primitives[node['id']]['recall_threshold'] += increment
+                if len(nodes) == 1 and self.summarize_for_single_node:
+                    return {"recall_threshold": response['message']}
+                result[node['name']] = {"recall_threshold": response['message']}
             except Exception as exception:
-                self.failures.append({node_name: exception})
-                raise Exception("Query Failure:", {node_name: exception})
+                self.failures.append({node['name']: exception})
+                raise
 
         return result
 
@@ -464,24 +509,24 @@ class BottleClient:
     def ping(self, nodes: List = None) -> List[Dict[str, Any]]:
         """Ping a node to ensure it's up."""
         if not self._connected:
-            raise ConnectionError(
-                'Not connected to a bottle. You must call `connect()` on a BottleClient instance before making queries')
+            raise ConnectionError('Not connected to a bottle. You must call `connect()` on a BottleClient instance before making queries')
         if nodes is None:
             return requests.get(f'{self.url}bottle/ping', headers=self.headers).json()
-        if nodes:
-            results = []
+        else:
+            nodes = [node for node in self.all_nodes if (node['name'] in nodes)]
+            results = {}
             for node in nodes:
-                response = requests.get(f'{self.url}{self.genome.primitive_map[node]}/ping', verify=self.secure,
+                response = requests.get(f"{self.url}{node['id']}/ping", verify=self.secure,
                                         headers=self.headers).json()
-                if 'error' in response or response["message"] != 'okay':
-                    self.system_failures.append({node: response})
-                    print("System Failure:", {node["name"]: response})
+                if 'error' in response or response["status"] == 'failed':
                     if len(nodes) == 1 and self.summarize_for_single_node:
-                        return response['message']
-                results.append({node: response["message"]})
+                        raise Exception("Request Failure:", response)
+                    print("Failure:", {node['name']: response})
+                if len(nodes) == 1 and self.summarize_for_single_node:
+                    return response['message']
+                results[node['name']] = response["message"]
             return results
 
-
     def define_mood(self, mood_list: List, nodes: List = None) -> List:
         """
         *mood_data* should be of the format:
@@ -490,20 +535,23 @@ class BottleClient:
         where node-name is the *name* of a primitive or manipulative.
         """
         if not self._connected:
-            raise ConnectionError(
-                'Not connected to a bottle. You must call `connect()` on a BottleClient instance before making queries')
-        results = []
+            raise ConnectionError('Not connected to a bottle. You must call `connect()` on a BottleClient instance before making queries')
+        results = {}
         if nodes is None:
-            nodes = [node["name"] for node in self.all_nodes]
+            nodes = [node['name'] for node in self.all_nodes]
+        else:
+            nodes = [node for node in self.all_nodes if (node['name'] in nodes)]
         for node in nodes:
-            response = requests.post(f'{self.url}{self.genome.primitive_map[node]}/mood/define', verify=self.secure,
+            response = requests.post(f"{self.url}{node['id']}/mood/define", verify=self.secure,
                                      headers=self.headers, json={'data': mood_list}).json()
-            if 'error' in response or response["message"] != 'defined-mood':
-                self.system_failures.append({node: response})
-                print("System Failure:", {node: response})
+            if 'error' in response or response['status'] == 'failed':
                 if len(nodes) == 1 and self.summarize_for_single_node:
-                    return response['message']
-            results.append({node: response["message"]})
+                    raise Exception("Request Failure:", response)
+                self.failures.append({node['name']: exception})
+                print("Failure:", {node['name']: response})
+            if len(nodes) == 1 and self.summarize_for_single_node:
+                return response['message']
+            results[node['name']] = response["message"]
         return results
 
     def set_mood(self, mood_data: Dict, nodes: List = None) -> List:
@@ -514,20 +562,23 @@ class BottleClient:
         where node-name is the *name* of a primitive or manipulative.
         """
         if not self._connected:
-            raise ConnectionError(
-                'Not connected to a bottle. You must call `connect()` on a BottleClient instance before making queries')
-        results = []
+            raise ConnectionError('Not connected to a bottle. You must call `connect()` on a BottleClient instance before making queries')
+        results = {}
         if nodes is None:
-            nodes = [node["name"] for node in self.all_nodes]
+            nodes = [node['name'] for node in self.all_nodes]
+        else:
+            nodes = [node for node in self.all_nodes if (node['name'] in nodes)]
         for node in nodes:
-            response = requests.post(f'{self.url}{self.genome.primitive_map[node]}/mood/set', verify=self.secure,
+            response = requests.post(f"{self.url}{node['id']}/mood/set", verify=self.secure,
                                      headers=self.headers, json={'data': mood_data}).json()
-            if 'error' in response or response["message"] != 'set-mood':
-                self.system_failures.append({node: response})
-                print("System Failure:", {node: response})
-                if len(mood_data) == 1 and self.summarize_for_single_node:
-                    return response['message']
-            results.append({node: response["message"]})
+            if 'error' in response or response["status"] == 'failed':
+                if len(nodes) == 1 and self.summarize_for_single_node:
+                    raise Exception("Request Failure:", response)
+                self.failures.append({node['name']: response})
+                print("Failure:", {node['name']: response})
+            if len(nodes) == 1 and self.summarize_for_single_node:
+                return response['message']
+            results[node['name']] = response["message"]
         return results
 
     def analyze_kb(self, nodes: List = None) -> List[Dict[str, Any]]:
@@ -544,10 +595,7 @@ class BottleClient:
         Monitors the KB analysis progress kickstarted by analyze_kb.
         """
         if not self._connected:
-            raise ConnectionError(
-                'Not connected to a bottle. You must call `connect()` on a BottleClient instance before making queries')
-        results = []
-        
+            raise ConnectionError('Not connected to a bottle. You must call `connect()` on a BottleClient instance before making queries')
         response = requests.get(f'{self.url}kb/analysis-progress', verify=self.secure, json=[], headers=self.headers).json()
         return response
 
@@ -563,3 +611,5 @@ class BottleClient:
         if nodes is None:
             nodes = self.query_nodes
         return self._query(requests.post, 'clear-target-class', nodes=nodes)
+
+
